import axios, { type AxiosError } from 'axios'
import Router from 'next/router'
import status from 'http-status'
import * as api from '@/api'
import { toast } from 'react-toastify'

interface ErrorResponse {
  statusCode: number
  message: string
}

const throwError = (error: AxiosError<ErrorResponse>) => {
  toast.error(
    error.response?.data.message.toString() ?? error.response?.statusText,
  )
  return Promise.reject(error)
}

axios.interceptors.request.use(function (config) {
  const accessToken = localStorage.getItem('accessToken')
  config.headers.Authorization = `Bearer ${accessToken}`
  return config
})

axios.interceptors.response.use(
  function (response) {
    return response
  },
  async function (error: AxiosError<ErrorResponse>) {
    // fail to refresh token
    if (
      error.config &&
      error.config.url === '/api/users/me/token' &&
      error.config.method === 'put'
    ) {
      Router.push('/login')
      return throwError(error)
    }
    switch (error.response?.status) {
      case status.UNAUTHORIZED: {
        if (error.config?.url === '/api/users/token') {
          throw throwError(error)
        }
        const res = await api.user.refreshToken()
        localStorage.setItem('accessToken', res.data)
        return error.config && axios.request(error.config)
      }
      default: {
        return throwError(error)
      }
    }
  },
)

export default axios
